#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# mq_install::fixpack
#
# @summary The defined type `mq_install::fixoack` allows you to install specified fixpacks on your system.
# The fixpacks will be added on top of your software installation managed by [`ora_install::software`](./software).
# 
# The simples way to use it is:
# 
# ``` puppet
# mq_install::fixpack{'9.1.0.1':
#   source_locaation => '/nfs_share/software'
# }
# ```
# Based on this puppet code, puppet will install fixpack 9.1.0.1. Because the installation of the fixpack requires all MQ processes to be stopped, this code will stop **ALL** MQ managers when it finds it needs to install the fixpack. 
# 
# When the `mq_install::fixpack' is part of a larger set of Puppet code also managing MQ managers and queue's, that definition will restart the required MQ queue masnagers.
#
# @param title
#    The name of the class.
#
# @param [String] source_location
#    The location of the MQ software.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          source_location => '/software',
#          ...
#        }
#
# @param [String] tmp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          tmp_dir => '/my_tmp_dir',
#          ...
#        }
#    On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [Variant[Enum['on_failure'], Boolean]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          logoutput => true,
#          ...
#        }
#
#--++--
define mq_install::fixpack(
  String                               $source_location,
  String                               $tmp_dir            = '/tmp',          # Directory used for unpacking
  Variant[Enum['on_failure'], Boolean] $logoutput          = 'on_failure',    # Output of the execs
)
{
  # assert_type($title, Pattern[/\d+\.\d+\.\d+\.\d/] )
  $version = $title

  if $::osfamily == 'windows' { fail 'This module does not support Windows'}
  mq_install { "mq_install::fixpack::${version}": }
  $version_components       = $version.split('\.')
  #
  # On the package version the last number is separated by an dash
  #
  $package_version = "U${version_components[0]}${version_components[1]}${version_components[2]}${version_components[3]}-${version_components[0]}.${version_components[1]}.${version_components[2]}-${version_components[3]}"
  $file_name       = "${version_components[0]}.${version_components[1]}.${version_components[2]}-IBM-MQ-LinuxX64-FP000${version_components[3]}"
  #
  # If the installed version is the same as the requested version. Do nothing
  #
  if $version != $::mq_version {
    file { "${tmp_dir}/mq_fixpack_${version}":
      ensure => 'directory',
    }

    -> archive { "${tmp_dir}/${file_name}.tar.gz":
      ensure       => present,
      extract      => true,
      extract_path => "${tmp_dir}/mq_fixpack_${version}",
      source       => "${source_location}/${file_name}.tar.gz",
      cleanup      => true,
    }

    #
    # MQ only allows us to install a fixpack package when no components
    # are running. So we have to stop all before continuing.
    #
    class { '::mq_install::internal::mq_stop':
      logoutput => $logoutput,
    }

    -> mq_install::internal::mq_packages { $package_version:
      packages        => $::mq_installed_rpms,
      source_location => "${tmp_dir}/mq_fixpack_${version}",
    }

    ->exec {"Remove temporary files for fixpack ${version}":
      path    => '/bin',
      command => "rm -rf ${tmp_dir}/mq_fixpack_${version}",
      onlyif  => "test -d ${tmp_dir}/mq_fixpack_${version}"
    }
    contain '::mq_install::internal::mq_stop'
  }

}
