#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# mq_install::software
#
# @summary This class install's specified version of the the MQ software on the system.
# The source zip file from IBM is found at the location specified at `source_location`. By default it will install only the minimal required set. By using the `install_...` parameters, you can install optional components.
#
# Here is an example on how to use it:
#
# ``` puppet
# class { '::mq_install::software':
#   source_location  => '/software',
#   version          => '9.0.0.0'
#   tmp_dir          => '/tmp'
#   logoutput        => 'true,
# }
# ```
#
# On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [String] source_location
#    The location of the MQ software.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          source_location => '/software',
#          ...
#        }
#
# @param [Pattern[/\d+\.\d+\.\d+\.\d/]] version
#    The version to be installed.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          version => '9.0.0.0',
#          ...
#        }
#
# @param [String] tmp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          tmp_dir => '/my_tmp_dir',
#          ...
#        }
#    On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#

class mq_install::software(
  String                               $source_location,
  Pattern[/\d+\.\d+\.\d+\.\d/]         $version,                              # Version of the MQ software to install
  String                               $tmp_dir            = '/tmp',          # Directory used for unpacking
  Boolean                              $install_ams        = false,           # Install AMS packages
  Boolean                              $install_ft         = false,           # Install FT packages
  Boolean                              $install_client     = false,           # Install Client packages
  Boolean                              $install_explorer   = false,           # Install Explorer packages
  Boolean                              $install_xrserver   = false,           # Install XRServer packages
  Boolean                              $install_samples    = false,           # Install Samples packages
  Boolean                              $install_sdk        = false,           # Install SDK packages
  Boolean                              $install_man        = false,           # Install man packages
  Boolean                              $install_gskit      = false,           # Install GSKit packages
  Boolean                              $install_amqp       = false,           # Install AMQP packages
  Boolean                              $install_web        = false,           # REST API and Console
  Boolean                              $install_salesforce = false,           # IBM MQ Bridge to Salesforce
  Boolean                              $install_blockchain = false,           # IBM MQ Bridge to blockchain
  Boolean                              $install_rdqm       = false,           # replicated data queue manage
  Boolean                              $install_toolkit    = false,           # Install Development Toolkit packages
  Variant[Enum['on_failure'], Boolean] $logoutput          = 'on_failure',    # Output of the execs
)
{
  if $::osfamily == 'windows' {
    if $install_samples    { err 'Module Samples packages is not supported.' }
    if $install_sdk        { err 'Module SDK packages is not supported.' }
    if $install_man        { err 'Module man packages is not supported.' }
    if $install_gskit      { err 'Module GSKit packages is not supported.' }
    if $install_salesforce { err 'Module IBM MQ Bridge to Salesforce is not supported.' }
    if $install_blockchain { err 'Module IBM MQ Bridge to blockchain is not supported.' }
    if $install_rdqm       { err 'Module replicated data queue manage is not supported.' }

    if $install_samples or $install_sdk or $install_man or $install_gskit or $install_salesforce or $install_blockchain or $install_rdqm {
      fail 'Listed modules are not supported in Windows IBM MQ Version'
    }
    mq_install::software::windows{ "windows_installation_${version}":
      source_location  => $source_location,
      version          => $version,
      tmp_dir          => $tmp_dir,
      install_ams      => $install_ams,
      install_ft       => $install_ft,
      install_client   => $install_client,
      install_explorer => $install_explorer,
      install_xrserver => $install_xrserver,
      install_amqp     => $install_amqp,
      install_web      => $install_web,
      install_toolkit  => $install_toolkit,
      logoutput        => $logoutput
    }
  }
  else {
    if $install_toolkit # install_toolkit is only module not supported by linux version.
    {
      err 'Module Development Toolkit packages is not supported'
      fail 'Listed modules are not supported in Linux IBM MQ Version'
    }

    mq_install::software::linux{ "linux_installation_${version}":
      source_location    => $source_location,
      version            => $version,
      tmp_dir            => $tmp_dir,
      install_ams        => $install_ams,
      install_ft         => $install_ft,
      install_client     => $install_client,
      install_explorer   => $install_explorer,
      install_xrserver   => $install_xrserver,
      install_samples    => $install_samples,
      install_sdk        => $install_sdk,
      install_man        => $install_man,
      install_gskit      => $install_gskit,
      install_amqp       => $install_amqp,
      install_web        => $install_web,
      install_salesforce => $install_salesforce,
      install_blockchain => $install_blockchain,
      install_rdqm       => $install_rdqm,
      logoutput          => $logoutput
    }
  }
}
