#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# mq_install::software
#
# @summary This class install's specified version of the the MQ software on the system.
# The source zip file from IBM is found at the location specified at `source_location`. By default it will install only the minimal required set. By using the `install_...` parameters, you can install optional components.
#
# Here is an example on how to use it:
#
# ``` puppet
# class { '::mq_install::software':
#   source_location  => '/software',
#   version          => '9.0.0.0'
#   tmp_dir          => '/tmp'
#   install_ams      => true,
#   install_ft       => false,
#   install_client   => false,
#   install_explorer => true,
#   install_xrserver => false,
#   install_samples  => false,
#   install_sdk      => false,
#   install_man      => false,
#   install_gskit    => false,
#   install_amqp     => false,
#   logoutput        => 'true,
# }
# ```
#
# On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [String] source_location
#    The location of the MQ software.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          source_location => '/software',
#          ...
#        }
#
# @param [Pattern[/\d+\.\d+\.\d+\.\d/]] version
#    The version to be installed.
#    Here is an example on how to use this:
#        class { '::mq_install::software':
#          ...
#          version => '9.0.0.0',
#          ...
#        }
#
# @param [String] tmp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          tmp_dir => '/my_tmp_dir',
#          ...
#        }
#    On systems with a secured `/tmp` direcory, you MUST specify the `tmp_dir` parameter and specify a directory that puppet is allowed to execute scripts from. It must also have enough space to receive the extracted MQ installation kit.
#
# @param [Boolean] install_ams
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_ams => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_ft
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_ft => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_client
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_client => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_explorer
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_explorer => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_xrserver
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_xserver => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_samples
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_samples => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_sdk
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_sdk => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_man
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_man => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_gskit
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_gskit => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_amqp
#    Specify a value of `true` when you want to install this component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_amqp => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#
# @param [Boolean] install_web
#
#
# @param [Boolean] install_salesforce
#    Specify a value of `true` when you want to install the Bridge to Salesforce component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_salesforce => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to configure the connections to Salesforce and IBM MQ, then run the runmqsfb command to subscribe to events from Salesforce and publish them to an IBM MQ network.
#
# @param [Boolean] install_blockchain
#    Specify a value of `true` when you want to install the IBM MQ Bridge to blockchain component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_blockchain => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to configure the connections to your blockchain network and IBM MQ. You can then run the runmqbcb command to start the bridge and send queries and updates to, and receive responses from your blockchain network. The queue manager that the bridge connects to must be an IBM MQ Advanced queue manager.
#    This component is available only on Linux for System x (64 bit), on RHEL 7.3 or later and is only supported for MQ 9.1 and higher.
#
# @param [Boolean] install_rdqm
#    Specify a value of `true` when you want to install the replicated data queue manager component.
#    Here is an example:
#        class { '::mq_install::software':
#          ...
#          install_rqdm => true,
#          ...
#        }
#    The default is 'false'. Meaning this component is not installed.
#    Install this component to make the replicated data queue manager high availability configuration available. See Installing RDQM (replicated data queue managers) for more information.
#    This component is available only on Linux for System x (64 bit), on RHEL 7.3 or later and is only supported for MQ 9.1 and higher.
#
# @param [Variant[Enum['on_failure'], Boolean]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#        class { '::mq_install::...':
#          ...
#          logoutput => true,
#          ...
#        }
#
#--++--
define mq_install::software::windows(
  String                               $source_location,
  Pattern[/\d+\.\d+\.\d+\.\d/]         $version,                              # Version of the MQ software to install
  String                               $tmp_dir            = 'C:\\tmp',       # Directory used for unpacking
  String                               $file_name          = 'IBM_MQ_9.1_WINDOWS_TRIAL.zip', # Name of install file
  Boolean                              $install_ams        = false,           # Install AMS packages
  Boolean                              $install_ft         = false,           # Install FT packages
  Boolean                              $install_client     = false,           # Install Client packages
  Boolean                              $install_explorer   = false,           # Install Explorer packages
  Boolean                              $install_xrserver   = false,           # Install XRServer packages
  Boolean                              $install_amqp       = false,           # Install AMQP packages
  Boolean                              $install_web        = false,           # Install Web Administration packages
  Boolean                              $install_toolkit    = false,           # Install Development Toolkit packages
  Variant[Enum['on_failure'], Boolean] $logoutput          = 'on_failure',    # Output of the execs
)
{
  mq_install { 'setmq_install::software': }
  #
  # Setup some variables
  #
  $supported_major_versions = ['9']    # At this point in time only versio 8 is tested. Probably other versions work as well.
  $version_components       = $version.split('\.')
  #
  # On the package version the last number is separated by an dash
  #
  $major_version            = $version_components[0]
  $package_version          = "${version_components[0]}.${version_components[1]}.${version_components[2]}-${version_components[3]}"

  unless $major_version in $supported_major_versions {
    fail "MQ major version ${major_version} not (yet) supported. Only versions ${supported_major_versions.join(',')} supported."
  }
  $selected_packages = mq_install::selected_packages(
    [
      'JavaMsg',
      'JRE',
      'Server',
      # 'Toolkit', # toolkit is commented because it does install automatic anyway, but later it is seen as Development Toolkit, and parsed into Developed. So I have diecided not to manage it
      'Explorer',
      'Web',
    ],[
      $install_ams,
      $install_ft,
      $install_client,
      $install_xrserver,
      $install_amqp,
    ],[
      'AMS',
      ['MFT_Base', 'MFT_Agent','MFT_Logger','MFT_Service','MFT_Tools'],
      'Client',
      'XR_Service',
      'AMQP',
    ])

  $packages_to_add    = $selected_packages - $::mq_installed_components
  $packages_to_remove = $::mq_installed_components - $selected_packages - ['Development'] # comment line 248
  if $packages_to_add != [] or $packages_to_remove != []{

    unless defined(File[$tmp_dir]) {
      file { $tmp_dir:
        ensure => 'directory',
      }
    }

    archive { "${tmp_dir}\\${file_name}":
      ensure       => present,
      extract      => true,
      extract_path => $tmp_dir,
      source       => "${source_location}\\${file_name}",
      creates      => "${tmp_dir}\\MQServer",
      cleanup      => true,
      require      => File[$tmp_dir],
    }

    ->file {"${tmp_dir}\\MQServer":
      ensure  => 'directory',
    }

    -> file { "${tmp_dir}\\MQServer\\Response.ini":
      ensure  => file,
      content => epp('mq_install/response.ini.epp',{'add' => $packages_to_add, 'remove' =>$packages_to_remove }),
      backup  => false,
    }
    -> package { 'IBM MQ(Installation1)':
      ensure          => installed,
      source          => "${tmp_dir}\\MQServer\\MSI\\IBM MQ.msi",
      install_options => ["USEINI=\"${tmp_dir}\\MQServer\\Response.ini\"", 'AGREETOLICENSE="yes"','/l*v', "${tmp_dir}\\mqserver_install.log"],
    }

    #
    # Because the mq_config module start's right away, we neede to make sure the IBM MQ task 
    #  is up and running before leaving this module.
    #
    ~> exec {'wait for IBM Service manager':
      command     => epp('mq_install/wait_for_service_manager.ps1.epp'),
      refreshonly => true,
      provider    => 'powershell',
      timeout     => 60,
    }

    exec {'Remove temporary MQ files':
      path    => 'c:\\windows\\system32',
      command => "cmd.exe /c \"rmdir /s /q ${tmp_dir}\\MQServer\"",
      require => Package['IBM MQ(Installation1)'],
    }

  }
}
